@@ -1,6 +1,6 @@
 # pylint: disable=wildcard-import
 
-__version__ = (9, 0, 9792)
+__version__ = (9, 0, 10281)
 
 if bytes is str:
     raise Exception("""
@@ -26,6 +26,11 @@ loggers = Loggers()
 del Loggers
 del logging
 
+# import hook: pkg_resources is too slow to import, but it is used by some other packages that angr depends on. while
+# we upstream our fixes, we replace it with a light-weight solution.
+from .misc.import_hooks import import_fake_pkg_resources
+import_fake_pkg_resources(force=False)
+
 # this must happen first, prior to initializing analyses
 from .sim_procedure import SimProcedure
 from .procedures import SIM_PROCEDURES, SimProcedures, SIM_LIBRARIES
