@@ -11,6 +11,11 @@ from .bool_expr_xor_1 import BoolExprXor1
 from .remove_redundant_bitmasks import RemoveRedundantBitmasks
 from .remove_redundant_nots import RemoveRedundantNots
 from .remove_redundant_shifts import RemoveRedundantShifts
+from .simplify_pc_relative_loads import SimplifyPcRelativeLoads
+from .basepointeroffset_add_n import BasePointerOffsetAddN
+from .basepointeroffset_and_mask import BasePointerOffsetAndMask
+from .remove_redundant_conversions import RemoveRedundantConversions
+from .remove_cascading_conversions import RemoveCascadingConversions
 from .conv_shl_shr import ConvShlShr
 
 from .base import PeepholeOptimizationExprBase, PeepholeOptimizationStmtBase
