@@ -77,7 +77,7 @@ class SimEnginePropagatorAIL(
         data = self._expr(stmt.data)
 
         # is it accessing the stack?
-        sp_offset = self.extract_offset_to_sp(addr.one_expr)
+        sp_offset = self.extract_offset_to_sp(addr.one_expr) if addr.one_expr is not None else None
         if sp_offset is not None:
             if isinstance(data.one_expr, Expr.StackBaseOffset):
                 # convert it to a BV
