@@ -1152,7 +1152,7 @@ class Function(Serializable):
                 blocks.append(b)
                 block_addr_to_insns[b.addr] = sorted(common_insns)
 
-       #subgraph = networkx.subgraph(self.graph, blocks)
+        #subgraph = networkx.subgraph(self.graph, blocks)
         subgraph = self.graph.subgraph(blocks).copy()
         g = networkx.DiGraph()
 
@@ -1454,7 +1454,7 @@ class Function(Serializable):
         if self.name[0:2] == "_Z":
             try:
                 ast = parse(self.name)
-            except NotImplementedError:
+            except (NotImplementedError, KeyError):  # itanium demangler is not the most robust package in the world
                 return self.name
             if ast:
                 return ast.__str__()
