@@ -5,7 +5,6 @@ import ctypes
 import cffi # lmao
 import threading
 import itertools
-import pkg_resources
 import logging
 import pyvex
 import claripy
@@ -307,6 +306,23 @@ class _VexArchInfo(ctypes.Structure):
         ("x86_cr0", ctypes.c_uint),
     ]
 
+
+def _locate_lib(module: str, library: str) -> str:
+    """
+    Attempt to find a native library without using pkg_resources, and only fall back to pkg_resources upon failures.
+    This is because "import pkg_resources" is slow.
+
+    :return:    The full path of the native library.
+    """
+    base_dir = os.path.join(os.path.dirname(__file__), "..")
+    attempt = os.path.join(base_dir, library)
+    if os.path.isfile(attempt):
+        return attempt
+
+    import pkg_resources  # pylint:disable=import-outside-toplevel
+    return pkg_resources.resource_filename(module, os.path.join('lib', library))
+
+
 def _load_native():
     if sys.platform == 'darwin':
         libfile = 'angr_native.dylib'
@@ -316,7 +332,7 @@ def _load_native():
         libfile = 'angr_native.so'
 
     try:
-        angr_path = pkg_resources.resource_filename('angr', os.path.join('lib', libfile))
+        angr_path = _locate_lib("angr", os.path.join("lib", libfile))
         h = ctypes.CDLL(angr_path)
 
         VexArch = ctypes.c_int
